@@ -102,7 +102,9 @@ plot.keras_training_history <- function(x, y, metrics = NULL, method = c("auto",
       p <- ggplot2::ggplot(df, ggplot2::aes_(~epoch, ~value))
     }
 
-    smooth_args <- list(se = FALSE, method = 'loess', na.rm = TRUE)
+    smooth_args <- list(se = FALSE, method = 'loess', na.rm = TRUE,
+                        formula = y ~ x)
+    environment(smooth_args$formula) <- baseenv()
 
     if (theme_bw) {
       smooth_args$size <- 0.5
