@@ -155,10 +155,12 @@ new_layer_wrapper <- function(py_obj) {
 
 
 print.r_py_wrapper2 <- function(x, ...) {
-  clipr::write_clip(x)
+  try(clipr::write_clip(x))
   cat(x)
 }
 
 ## example usage:
 # new_layer_wrapper(keras$layers$TextVectorization)
 # new_layer_wrapper(keras$layers$DepthwiseConv1D)
+# new_layer_wrapper(keras$layers$UnitNormalization)
+new_layer_wrapper(keras$layers$RandomBrightness)
